@@ -18,22 +18,28 @@ if [ ! -w memtest.bin ]; then
 	exit 1
 fi
 
+
 # enlarge the size of memtest.bin
 SIZE=$(wc -c memtest.bin | awk '{print $1}')
 FILL=$((1474560 - $SIZE))
 dd if=/dev/zero of=fill.tmp bs=$FILL count=1
-cat memtest.bin fill.tmp >memtest.img
+cat memtest.bin fill.tmp > memtest.img
 rm -f fill.tmp
 
-echo "generating iso image ..."
+echo "Generating iso image ..."
 
 mkdir "cd"
 mkdir "cd/boot"
 mv memtest.img cd/boot
 cd cd
-mkisofs -b boot/memtest.img -c boot/boot.catalog -o memtest.iso .
-mv memtest.iso ..
+
+# Create the cd.README
+echo -e "There is nothing to do here\r\r\nMemtest86+ is located on the bootsector of this CD\r\r\n" > README.TXT
+echo -e "Just boot from this CD and Memtest86+ will launch" >> README.TXT
+
+mkisofs -A "MKISOFS 1.1.2" -p "Memtest86+ 2.01" -publisher "Samuel D. <memtest@memtest.org>" -b boot/memtest.img -c boot/boot.catalog -V "MT201" -o memtest.iso .
+mv memtest.iso ../mt201.iso
 cd ..
 rm -rf cd
 
-echo "done"
+echo "Done! Memtest86+ ISO is mt201.iso"
